#include <EEPROM.h>
#include <PID_Beta6.h>
#include <adc.h>
#include <display.h>
#include <fet.h>
#include <keypad.h>
#include <pressure.h>
#include <servo.h>
#include <temp.h>
#include <timer.h>
#include <ui.h>
#include <util.h>
#include <avr/io.h>    

#define ALARM_FET FET6
#define EJECTOR_FET FET7

#define GRATE_OFF 0
#define GRATE_ON 1
#define GRATE_PRATIO 2

//Datalogging variables
int lineCount = 0;

// Grate turning variables
int GRATE_SOLENOID = FET5;
//int GRATE_MOTOR_0 = FET7;
//int GRATE_MOTOR_1 = FET6;
unsigned long gratePeriod = 300000;
int gratePulseLength = 30000;
int grateMode = GRATE_PRATIO;
unsigned long nextGrate;
float pRatio, pRatioSmooth;
boolean pRatioHigh;
boolean grateOn;

// Flow variables
float CfA0_air_rct =0.42123;
float CfA0_gas = 0.81046;
double air_rct_flow;
double gas_flow;

int loopPeriod0 = 5000;
unsigned long nextTime0;
int loopPeriod1 = 1000;
unsigned long nextTime1;
int loopPeriod2 = 100;
unsigned long nextTime2;
int loopPeriod3 = 10;
unsigned long nextTime3;

//Ejector variables
double ejector_setpoint, ejector_input, ejector_output;
PID ejector_PID(&ejector_input, &ejector_output, &ejector_setpoint,0,0,0);
int ejector_period = 1000;
unsigned long ejector_nextCompute = 0;
double ejector_P = -15;
double ejector_I = 10;
double ejector_D = 0;

//Hertz
int hertz = 0;

// Pressure variables
int Press_Calib[6];
int Press[6]; //values corrected for sensor offset (calibration)
int P_air_in;
int P_gas_out;
int P_comb;
float P_comb_smooth;
int P_reactor;

//Servo
int servo_alt = 0; //used to pulse every other time through loop (~20 ms)

//Servo0
float servo0_pos = 0;
float servo0_db = 0; // used to deadband the servo movement

//Servo1
float servo1_pos;
float servo1_db = 0; // used to deadband the servo movement

// Alarm
int auger_on_length = 0;
int auger_off_length = 0;
unsigned int auger_on_alarm_point = 300;
unsigned int auger_off_alarm_point = 900;
boolean alarm;
int LOW_FUEL_TC = 3;
int alarm_interval = 5; // in seconds

void setup() {
  GCU_Setup(V3,FULLFILL,P777222);
  //
  DDRJ |= 0x80;      
  PORTJ |= 0x80;   
  delay(1);	
  
  // timer initialization
  nextTime0 = millis() + loopPeriod0;
  nextTime1 = millis() + loopPeriod1;
  nextTime2 = millis() + loopPeriod2;
  nextTime3 = millis() + loopPeriod3;

  Ejector_Init(); // params should be pulled out to here
  LoadPressureSensorCalibration();
  //LoadLambda(); - must save lambda data first?
  Serial.begin(57600);

  Disp_Init();
  Kpd_Init();
  UI_Init();
  ADC_Init();
  Temp_Init();
  Press_Init();
  Fet_Init();
  Servo_Init();
  Timer_Init();

  Disp_Reset();
  Kpd_Reset();
  UI_Reset();
  ADC_Reset();
  Temp_Reset();
  Press_Reset();
  Fet_Reset();
  Servo_Reset();
  Timer_Reset();
}

void loop() {
  int key;
  if (millis() >= nextTime3) {
    nextTime3 += loopPeriod3;
    // first, read all KS's sensors
    Temp_ReadAll();  // reads into array Temp_Data[], in deg C
    Press_ReadAll(); // reads into array Press_Data[], in hPa
    Timer_ReadAll(); // reads pulse timer into Timer_Data, in RPM ??? XXX
    UpdateCalibratedPressure();
    DoPressure();
    //DoFlow();
    DoSerialIn();
    DoEjector();
    DoServos();
    if (millis() >= nextTime2) {
      //MeasureElectricalPower();
      //accumulateEnergyValues();
      if (millis() >= nextTime1) {
        nextTime1 += loopPeriod1;
        //averageEnergyValues();
        //DoHertz();
        DoGrate();
        DoDatalogging();
        DoAlarmUpdate();
        grateOn = false;
        if (alarm == true) {
          analogWrite(ALARM_FET, 255);
        } else {
          analogWrite(ALARM_FET,0);
        }
        if (millis() >= nextTime0) {
          nextTime0 += loopPeriod0;
          DoAlarm();
        }
      }
    }
    // END USER CONTROL CODE
    UI_DoScr();       // output the display screen data, 
    // (default User Interface functions are in library KS/ui.c)
    // XXX should be migrated out of library layer, up to sketch layer                      
     key = Kpd_GetKeyAsync();
    // get key asynnchronous (doesn't wait for a keypress)
    // returns -1 if no key

    UI_HandleKey(key);  // the other two thirds of the UI routines:
    // given the key press (if any), then update the internal
    //   User Interface data structures
    // ALSO: Manipulate the various output data structures
    //   based on the keypad input

    Fet_WriteAll();   // Write the FET output data to the PWM hardware
    Servo_WriteAll(); // Write the Futaba hobby servo data to the PWM hardware

    PORTJ ^= 0x80;    // toggle the heartbeat LED
  }
}

